#include "BF.h"

#include <stdio.h>
#include <map>

namespace BF
{
    const double BF::BF_EPS = 1e-10;
    
    BF::BF(int _numFreq, int _samplingRate, double _soundSpeed) : numFreq(_numFreq), samplingRate(_samplingRate), soundSpeed(_soundSpeed), transformFunctionDatabase(NULL)
    {
    }

    BF::~BF()
    {
	delete transformFunctionDatabase;
    }

    void
    BF::getTransformFunctions(const std::vector<SourceLocation> &sourcePositions, std::vector<Matrix> &transformFunctions) const
    {
		int numSrc = sourcePositions.size();
		std::vector<Matrix> newTransformFunctions;
		std::vector<SourceLocation> newSourcePositions;


	    /* 伝達関数をデータベースから取得する場合、マイク数はデータベースから取得する。 */
	    int numMic = this->transformFunctionDatabase->micLocations.size();

	    /* 伝達関数をデータベースから取得するが、まずは取得結果を覚める領域を確保 */
	    for(int i = 0; i < this->numFreq; i++){
			newTransformFunctions.push_back(Matrix(numMic, numSrc));
	    }
	    /* 続いて、入力の音源・マイクの組を、データベース上どの音源・マイクの組にマップするかを検索 */
	    std::map<int, int> srcMap, micMap;
	    for(int i = 0; i < numSrc; i++){
			int srcIndex = this->transformFunctionDatabase->findNearestSourceIndex(sourcePositions[i]);
			srcMap.insert(std::pair<int, int>(i, srcIndex));
	    }

	    for(int i = 0; i < this->numFreq; i++){
			Matrix &transformFunction = newTransformFunctions[i];
			for(int j = 0; j < numSrc; j++){
			    int jj = srcMap[j];
			    transformFunction.setColumn(j, this->transformFunctionDatabase->transformFunctions[i].getColumn(srcMap[j]));
			}
	    }

		/* 結果の格納 */
		newTransformFunctions.swap(transformFunctions);
    }

    void
    BF::executeBeamForming(const Matrix &inputSpectrum, Matrix &outputSpectrum)
    {
	Matrix newInputSpectrum (inputSpectrum);
	Matrix newOutputSpectrum(this->separationMatrices[0].getRowCount(), this->numFreq);

	for(int i = 0; i < this->numFreq; i++){
//	    this->separationMatrices[i].dump(stdout, "this->separationMatrices[%d]", i);
//	    inputSpectrum.getColumn(i).dump(stdout, "inputSpectrum.getColumn(%d)", i);
//	    this->separationMatrices[i].multiplied(inputSpectrum.getColumn(i)).dump(stdout, "this->separationMatrices[%d].multiplied(inputSpectrum.getColumn(%d))", i, i);
	    newOutputSpectrum.setColumn(i, this->separationMatrices[i].multiplied(inputSpectrum.getColumn(i)));
	}

//	inputSpectrum.dump(stdout, "in");
//	separationMatrices[0].dump(stdout, "G[0]");
//	newOutputSpectrum.dump(stdout, "out");

	outputSpectrum = newOutputSpectrum;
	newInputSpectrum.swap(this->inputSpectrum);
	newOutputSpectrum.swap(this->outputSpectrum);
    }

    void
    BF::setTransformFunctionDatabase(const TransformFunctionDatabase &transformFunctionDatabase)
    {
	using std::swap;
	TransformFunctionDatabase *newTransformFunctionDatabase = new TransformFunctionDatabase(transformFunctionDatabase);
	swap(this->transformFunctionDatabase, newTransformFunctionDatabase);
	delete newTransformFunctionDatabase;
    }
}
